/*
 * eximstate: Logging
 * ------------------
 *
 * Logging functions used in the client and server.
 *
 * (c) 2002 Claranet UK Ltd. by Ollie Cook <ollie@olliecook.net>
 *
 * $Id: logging.c,v 1.3 2002/11/19 19:36:56 ollie Exp $
 */
#if HAVE_CONFIG_H
#include "../config.h"
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#include <stdio.h>
#include <syslog.h>
#include <stdarg.h>
#include "eximstate.h"
#include "logging.h"

/* Function:    error
 * Description: Logs an error to the log file and to stderr
 */

void error (char *fmt, ... ) {
	char buf[MAX_BUF];
	va_list ap;

	va_start (ap, fmt);
	vsnprintf (buf, MAX_BUF - 1, fmt, ap);
	va_end (ap);

	syslog (LOG_ERR, buf);
	fprintf (stderr, "ERROR: ");
	fprintf (stderr, buf);
	fprintf (stderr, "\n");
	fflush (stderr);
}

/* Function:    log
 * Description: Logs an entry
 */

void log (char *fmt, ... ) {
	char buf[MAX_BUF];
	va_list ap;

	va_start (ap, fmt);
	vsnprintf (buf, MAX_BUF - 1, fmt, ap);
	va_end (ap);

	syslog (LOG_INFO, buf);

	if (debugflag == 1) {
		fprintf (stderr, "LOG:   ");
		fprintf (stderr, buf);
		fprintf (stderr, "\n");
		fflush (stderr);
	}
}

/* Function:    Debug
 * Description: Prints a message to standard error if the global debugging flag is set
 */

void debug (char *fmt, ... ) {
	char buf[MAX_BUF];
	va_list ap;

	if (debugflag == 1) {
		va_start (ap, fmt);
		vsnprintf (buf, MAX_BUF - 1, fmt, ap);
		va_end (ap);

		fprintf (stderr, "DEBUG: ");
		fprintf (stderr, buf);
		fprintf (stderr, "\n");
		fflush (stderr);
	}
}
