/*
 * eximstate: Network Functions
 * ----------------------------
 *
 * Network functions that are common to both server and client
 *
 * (c) 2002 Claranet UK Ltd. by Ollie Cook <ollie@olliecook.net>
 *
 * $Id: net.c,v 1.3 2002/11/19 19:36:56 ollie Exp $
 */

#if HAVE_CONFIG_H
#include "../config.h"
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include "config.h"

/* Function:    send_to_peer
 * Description: Sends data to peer at end of socket 'fd'
 * Returns:     Return code from send(), usually the
 *              number of characters sent
 */

int send_to_peer (int fd, char *fmt, ... ) {
	char buf[MAX_BUF];
	va_list ap;
	int rc;

	va_start (ap, fmt);
	vsnprintf (buf, MAX_BUF - 1, fmt, ap);
	va_end (ap);

	rc = send (fd, buf, strlen (buf), 0);

	return (rc);
}

/* Function:    read_line
 * Description: Reads a line until CRLF received into buf which must be previously allocated
 * Returns:     1 on success
 *              0 on error in which case the contents of
 *                buf is not known or guaranteed
 */

int read_line (char *buf, int fd) {
	int read_line = 0;      /* Denotes if we've read a whole line */
	size_t recvd_bytes = 0; /* Number of bytes received from server in this recv() */
	size_t line_len = 0;    /* Number of bytes received from server so far in this line */
	char tmp[MAX_BUF];

	bzero (buf, MAX_BUF);

	while (read_line == 0) {
		recvd_bytes = recv (fd, tmp, sizeof (tmp) - 1, 0);
		if (recvd_bytes == -1) {
			if (errno == EAGAIN)
				continue;
			else 
				return (0);
		} 
		line_len += recvd_bytes;
		strncat (buf, tmp, sizeof (tmp));
		if (buf[line_len - 2] == '\r' && buf[line_len - 1] == '\n') {
			read_line = 1;
		}
	}

	return (1);
}
