/*
 * eximstate: RRDtool argv tool
 * ----------------------------
 *
 * Convert char** to argv for rrdtool_ functions
 *
 * (c) 2002 Claranet UK Ltd. by Ollie Cook <ollie@olliecook.net>
 *
 * $Id: rrd_argv.c,v 1.2 2002/11/19 19:36:56 ollie Exp $
 */ 

#if HAVE_CONFIG_H
#include "../config.h"
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <errno.h>
#include <stdio.h>
#include <rrd.h>
#include "config.h"
#include "rrd_argv.h"

/* Function:    string_to_argv 
 * Description: Allocates memory for argv and parses each argument
 *              in argstring to be an element in argv
 * Returns:     pointer to argv or NULL on errorS
 *
 * Based on http://people.ee.ethz.ch/~oetiker/webtools/rrdtool/pub/contrib/rrd_wrapper.zip by
 * Antony Clothier
 */
char **string_to_argv(char *argstring, int *argc) {
	char inquotes = 0, *workstring, **argv;
	int i, nchars;

	if (!argstring) {
		errno = EINVAL;
		return (NULL);
	}

	nchars = strlen (argstring);
	workstring = (char *) calloc (nchars + 1, sizeof (char *));

	/* fill the array with space characters */
	for(i=0; i < (nchars + 1); i++)
		workstring[i] = ' '; 

	/* copy the argstring into the workstring (padded with spaces on both ends) */
	for(i=0; i < nchars; i++)
		workstring[i] = argstring[i]; 

	/* move through workstring counting arguments */
	for((*argc) = 1, i=0; i < nchars + 1; i++) {
		if(isgraph (workstring[i])) {
			/* if current character is not whitespace, previous character was whitespace
			 * and we're not part of a quoted string, increment (*argc) */
			if((!isgraph (workstring[i-1])) && (inquotes == 0))
				(*argc)++;

			/* check for the quote character and toggle inquotes */
			if(workstring[i] == '"') {
				inquotes = (char)(!inquotes);
			}
		} else {
			/* convert whitespace to NULL characters for separation */
			if(!inquotes)
				workstring[i] = '\0';
		}
	}

	/* If we've counted no arguments return NULL */
	if((*argc) == 0) {
		free (workstring);
		errno = EINVAL;
		return (NULL);
	} else {
		inquotes = 0;

		argv = (char**) calloc (*argc, sizeof (char**));
		argv[0] = &workstring[0];

		/* Work through workstring again, copying it to argv */
		for((*argc) = 1, i=1; i < nchars + 1; i++) {
			if(isgraph (workstring[i])) {
				if((!isgraph (workstring[i-1])) && (inquotes == 0)) {
					argv[(*argc)] = &workstring[i];
					(*argc)++;
				}

				/* check for the quote character and toggle inquotes */
				if(workstring[i] == '"') {
					inquotes = (char)(!inquotes);
				}
			}
		}
	}
	return (argv);
}

/* Function:    free_argv
 * Description: Frees memory allocated by string_to_argv
 */
void free_argv(char** argv) {
	free (argv[0]);
	free (argv);
}
